%SMsolver2
%			Solves for grounding line position for marine ice sheet with
%			grounding line flux given as per Schoof, 2007, Ice sheet
%			grounding line dynamics: steady states, stability and
%			hysteresis, JGR, 112, F03S28, doi:10.1029/2006JF000664.               
%
%           To run this program, specify your grid points as user_grid as
%           a column vector below, in ascending order. Also enter an initial
%           guess x for grounding line position. Default is for a fixed
%           grid in MISMIP mode 1
%
%           Output from he program is a grounding line position x and an
%           ice thickness user_thickness at all grid points user_grid. For grid
%           points < x, ice is grounded, otherwise floating.
%
%           Also ensure that the remaining physical parameters are entered
%           correctly below under `Parameters as defined in Schoof 2007...'; 
%           principally you will need to change A, m and C
%           The alternative values for m and C are given in commented-out
%           form below; A must be entered from the MISMIP draft in SI form
%
%           The depth to bed below sea level is defined in subroutine
%           SMcold_bedheight. DESPITE ITS NAME, this function computes
%           depth BELOW sea level, not height above, i.e. it returns b(x)
%           as in the MISMIP draft. SMcold_bedslope is the first derivative
%           of SMcold_bedheight, and must agree with SMcold_bedheight or
%           the Newton solver below may not converge; you will also get the
%           wrong surface elevation.
%
%           As default, SMcold_bedheight is set to b(x) given
%           by equation (1) in the MISMIP draft. The commented-out lines
%           in SMcold_bedheight and SMcold_bedslope beginning %z = ... give
%           the corresponding versions for b(x) given by equation (4) in
%           the MISMIP draft. Uncomment these as required.


%-----------------------------------------------------------------------
%Specify your computational grid
%-----------------------------------------------------------------------


user_grid = linspace(0,1.8e6,150);
            %user grid must be a COLUMN vector in metres


%-----------------------------------------------------------------------
%Parameters as defined in Schoof 2007, see also the MISMIP specifications
%-----------------------------------------------------------------------


global m n r A C rho_g a theta

n = 3;              %Glen's law exponent

m = 1;%1/3;         %sliding exponent

r = 0.9;            %ratio of ice to water density

rho_g = 8820;        %900 kg m^{-3} * 9.8 m s^{-2}

A = 4.6416e-24;               %Glen's law parameter in SI units
                    % NB Paterson (1994) does not use SI units!

                    
                    
%C = 7.624e6;        %m = 1/3 value
C = 7.2082e+010;     %m = 1 value 
                    %for both, tau_b = 80kPa gives u ~ 35 m a^(-1)

a = 0.3/(365*24*3600);

theta = 0;          %cold (theta = 1) or warm (theta = 0) boundary layer specifications,
                    %models A and B respectively in Schoof 2007

x = 1.270e6;        %initial guess of grounding line position (metres)


%-----------------------------------------------------------------------
%Newton iteration parameters
%-----------------------------------------------------------------------



delta_x = 10;       %finite difference step size (metres) for gradient calculation

tolf = 1e-4;        %tolerance for finding zeros
normf = tolf + eps;

toldelta = 1e1;     %Newton step size tolerance
dx = toldelta + 1;

%-----------------------------------------------------------------------
%Newton iteration
%-----------------------------------------------------------------------


delta_x;

while normf > tolf | abs(dx) > toldelta
    f = SMcold_function(x);
    normf = abs(f);
    grad = (SMcold_function(x+delta_x)-f)/delta_x;
    dx = -f/grad;
    x = x + dx;
end
                    %computes steady state grounding line position x based
                    %on intial guess in line 35, using equations (20) and
                    %(24) in Schoof 2007


%-----------------------------------------------------------------------
%Calculate ice surface elevation S_Soln for grounded sheet
%-----------------------------------------------------------------------


grounded = user_grid(user_grid < x).';

x_grid = [x; grounded(length(grounded):-1:1)];
                        %specifies the grid points on which  ice surface elevations are to be calculated
                        %these are in reverse order, i.e. counting
                        %backwards from the grounding line at x to the ice
                        %divide at zero. FIRST POINT MUST BE GROUNDING LINE
                        %POSITION x COMPUTED ABOVE, EVEN IF THIS IS NOT
                        %INCLUDED IN YOUR COMPUTATIONAL GRID

h_f = SMcold_bedheight(x)/r;
                        %computes ice thickness at the grounding line as
                        %initial condition

options = odeset('AbsTol',1e-6,'RelTol',1e-6);%odeset('AbsTol',f/1e-3);

[X_soln,H_soln] = ode45(@SMsurface,x_grid,[h_f],options);
                        %computes ice THICKNESS H_soln at points with
                        %position X_soln
                        
S_soln = H_soln - SMcold_bedheight(X_soln);
                        %computes ice surface elevation by adding bed
                        %elevation
                        
figure
hold on

plot(X_soln,-SMcold_bedheight(X_soln),'k')
plot(X_soln,S_soln,'b')

%H_soln is now rearranged and the point
%corresponding to the grounding line location is expunged to input these data into your
%computational grid:

S_soln = S_soln(length(S_soln):-1:2);

H_soln = H_soln(length(H_soln):-1:2);



%-----------------------------------------------------------------------
%Calculate ice thickness for shelf from van der Veen (1986)
%-----------------------------------------------------------------------



floating = user_grid(user_grid > x).';

q_0 = a*x;

H_soln2 = h_f*(q_0 + a*(floating-x))./(q_0^(n+1) + h_f^(n+1)*((1-r)*rho_g/4)^n*A*((q_0 + a*(floating-x)).^(n+1) - q_0^(n+1))/a).^(1/(n+1)); 

plot([x; floating],(1-r)*[h_f; H_soln2],'b');

plot([x; floating],-r*[h_f; H_soln2],'b');

user_thickness = [H_soln;H_soln2];